Package: meteoland
Type: Package
Title: Landscape Meteorology Tools
Version: 2.2.7
Date: 2026-05-08
Authors@R: c(
	person('Miquel', 'De Cáceres', role=c('aut','cre'),
	email='miquelcaceres@gmail.com', comment = c(ORCID = "0000-0001-7132-2080")),
	person('Víctor', 'Granda', role='aut', comment = c(ORCID = "0000-0002-0469-1991")),
	person('Nicolas', 'Martin', role='aut', comment = c(ORCID = "0000-0001-7574-0108")),
	person('Antoine', 'Cabon', role='aut', comment = c(ORCID = "0000-0001-6426-1726")))
Description: Functions to estimate weather variables at any position of a landscape [De Caceres et al. (2018) <doi:10.1016/j.envsoft.2018.08.003>].
License: GPL (>= 2)
URL: https://emf-creaf.github.io/meteoland/,
        https://github.com/emf-creaf/meteoland
BugReports: https://github.com/emf-creaf/meteoland/issues
LazyData: TRUE
Depends: R (>= 4.1.0)
Imports: methods, sf, stars, stats, Rcpp, units, lifecycle, cli, dplyr,
        tidyr, rlang, assertthat, purrr, ncdfgeom, ncmeta, lubridate,
        cubelyr
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), meteospain, worldmet,
        tibble
LinkingTo: Rcpp
Encoding: UTF-8
NeedsCompilation: yes
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-08 13:08:14 UTC; miquel
Author: Miquel De Cáceres [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7132-2080>),
  Víctor Granda [aut] (ORCID: <https://orcid.org/0000-0002-0469-1991>),
  Nicolas Martin [aut] (ORCID: <https://orcid.org/0000-0001-7574-0108>),
  Antoine Cabon [aut] (ORCID: <https://orcid.org/0000-0001-6426-1726>)
Maintainer: Miquel De Cáceres <miquelcaceres@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-08 14:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-08 16:11:37 UTC; unix
Archs: meteoland.so.dSYM
