# Generated by roxygen2: do not edit by hand

export(add_topo)
export(complete_meteo)
export(create_meteo_interpolator)
export(defaultInterpolationParams)
export(get_interpolation_params)
export(humidity_dewtemperature2relative)
export(humidity_relative2dewtemperature)
export(humidity_relative2specific)
export(humidity_specific2relative)
export(interpolate_data)
export(interpolation_cross_validation)
export(interpolation_dewtemperature)
export(interpolation_precipitation)
export(interpolation_temperature)
export(interpolation_wind)
export(interpolator_calibration)
export(meteospain2meteoland)
export(penman)
export(penmanmonteith)
export(precipitation_concentration)
export(precipitation_rainfall_erosivity)
export(radiation_dateStringToJulianDays)
export(radiation_daylength)
export(radiation_daylengthseconds)
export(radiation_directDiffuseDay)
export(radiation_directDiffuseInstant)
export(radiation_julianDay)
export(radiation_netRadiation)
export(radiation_outgoingLongwaveRadiation)
export(radiation_potentialRadiation)
export(radiation_skyLongwaveRadiation)
export(radiation_solarConstant)
export(radiation_solarDeclination)
export(radiation_solarElevation)
export(radiation_solarRadiation)
export(radiation_sunRiseSet)
export(read_interpolator)
export(set_interpolation_params)
export(summarise_interpolated_data)
export(summarise_interpolator)
export(utils_airDensity)
export(utils_atmosphericPressure)
export(utils_averageDailyVP)
export(utils_averageDaylightTemperature)
export(utils_latentHeatVaporisation)
export(utils_latentHeatVaporisationMol)
export(utils_psychrometricConstant)
export(utils_saturationVP)
export(utils_saturationVaporPressureCurveSlope)
export(with_meteo)
export(worldmet2meteoland)
export(write_interpolator)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(cubelyr,tbl_cube)
importFrom(dplyr,.data)
importFrom(lifecycle,deprecated)
importFrom(methods,.valueClassTest)
importFrom(rlang,":=")
importFrom(stats,aggregate)
importFrom(stats,complete.cases)
importFrom(stats,cor)
useDynLib(meteoland, .registration = TRUE)
